﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MonoBrickFirmware.Native;
using MonoBrickFirmware.Display;

namespace MonoBrickFirmware.HardwareIF
{
#if !Ev3Simu
#region Real Hardware
    public class LcdHAL
    {
        #region SingletonStuff
        // ensures that only one instance of this class can exist, so everybody writes and reads the same variables
        // http://geekswithblogs.net/BlackRabbitCoder/archive/2010/05/19/c-system.lazylttgt-and-the-singleton-design-pattern.aspx
        // static holder for instance, need to use lambda to construct since constructor private (DotNet4 only)
        private static readonly Lazy<LcdHAL> _instance = new Lazy<LcdHAL>(() => new LcdHAL());

        const int hwBufferLineSize = 60;
        const int hwBufferSize = hwBufferLineSize * Lcd.Height;
        private LcdHAL() // hiding creator
        {
            device = new UnixDevice("/dev/fb0");
            memory = device.MMap(hwBufferSize, 0);
        }  

        public static LcdHAL Init { get { return _instance.Value; } }
        
        #endregion //SingletonStuff

        byte[] hwBuffer = new byte[hwBufferSize];
        
        private UnixDevice device;
        private MemoryArea memory;

        static byte[] convert = 
		    {
		    0x00, // 000 00000000
		    0xE0, // 001 11100000
		    0x1C, // 010 00011100
		    0xFC, // 011 11111100
		    0x03, // 100 00000011
		    0xE3, // 101 11100011
		    0x1F, // 110 00011111
		    0xFF // 111 11111111
		    }; 

        public void Update(byte[] displayBuf, int yoffset = 0)
        {
            const int bytesPrLine = 3 * 7 + 2;
            int inOffset = (yoffset % Lcd.Height) * (bytesPrLine);
            int outOffset = 0;
            for (int row = 0; row < Lcd.Height; row++)
            {
                int pixels;
                for (int i = 0; i < 7; i++)
                {
                    pixels = displayBuf[inOffset++] & 0xff;
                    pixels |= (displayBuf[inOffset++] & 0xff) << 8;
                    pixels |= (displayBuf[inOffset++] & 0xff) << 16;
                    hwBuffer[outOffset++] = convert[pixels & 0x7];
                    pixels >>= 3;
                    hwBuffer[outOffset++] = convert[pixels & 0x7];
                    pixels >>= 3;
                    hwBuffer[outOffset++] = convert[pixels & 0x7];
                    pixels >>= 3;
                    hwBuffer[outOffset++] = convert[pixels & 0x7];
                    pixels >>= 3;
                    hwBuffer[outOffset++] = convert[pixels & 0x7];
                    pixels >>= 3;
                    hwBuffer[outOffset++] = convert[pixels & 0x7];
                    pixels >>= 3;
                    hwBuffer[outOffset++] = convert[pixels & 0x7];
                    pixels >>= 3;
                    hwBuffer[outOffset++] = convert[pixels & 0x7];
                }
                pixels = displayBuf[inOffset++] & 0xff;
                pixels |= (displayBuf[inOffset++] & 0xff) << 8;
                hwBuffer[outOffset++] = convert[pixels & 0x7];
                pixels >>= 3;
                hwBuffer[outOffset++] = convert[pixels & 0x7];
                pixels >>= 3;
                hwBuffer[outOffset++] = convert[pixels & 0x7];
                pixels >>= 3;
                hwBuffer[outOffset++] = convert[pixels & 0x7];
                if (inOffset >= Lcd.Height * bytesPrLine)
                    inOffset = 0;
            }
            memory.Write(0, hwBuffer);
        }

        public void Dispose() { device.Dispose(); }
    }
#endregion Real Hardware
#else
    #region Simulated Hardware
    public class LcdHAL
    {
    #region SingletonStuff
        // ensures that only one instance of this class can exist, so everybody writes and reads the same variables
        // http://geekswithblogs.net/BlackRabbitCoder/archive/2010/05/19/c-system.lazylttgt-and-the-singleton-design-pattern.aspx
        // static holder for instance, need to use lambda to construct since constructor private (DotNet4 only)
        private static readonly Lazy<LcdHAL> _instance = new Lazy<LcdHAL>(() => new LcdHAL());

        private LcdHAL()  // hiding creator
        {
            // nothing to do
        } 
                // accessor for instance
        public static LcdHAL Init { get { return _instance.Value; } }
 
    #endregion //SingletonStuff

        //public class Pixel 
        //{
        //    private bool _Pixel;
        //    public Pixel ss { get { return _Pixel;}}
        //}
        
        //public class PixelLine
        //{
        //    private bool[] _PixelLine = new bool[Lcd.Width];
        //    public bool[] PixelLine { 
        //        get { return _PixelLine;} 
        //        set {_PixelLine = value;} }
        //}

        //public class PixelArea
        //{
        //    private PixelLine[] _PixelArea
        //    {
        //        get { return _PixelArea; }
        //        set { _PixelArea = value; }
        //    }
        //}

        public byte[] CurrentLcd = new byte[(Lcd.Width + 7) / 8*Lcd.Height];

        public System.Drawing.Image SimuImage;

        public void Update(byte[] displayBuf, int yoffset = 0)
        {
            CurrentLcd = displayBuf;
        }

        public void Dispose()
        {
            // tbd
        }
    }



    #endregion Simulated Hardware
#endif

}
